@@ -122,11 +122,7 @@ class Command(BaseCommand):
             if project.enable_write_lockfile:
                 project.core.ui.info(f"Adding group [success]{group}[/] to lockfile")
             lock_groups.append(group)
-        if (
-            group == "default"
-            or not selection.dev
-            and group not in project.pyproject.settings.get("dev-dependencies", {})
-        ):
+        if group == "default" or not selection.dev and group not in project.pyproject.dev_dependencies:
             if editables:
                 raise PdmUsageError("Cannot add editables to the default or optional dependency group")
         for r in [parse_requirement(line, True) for line in editables] + [parse_requirement(line) for line in packages]:
